/********************************************************************
 globals.c : global data and functions for mps

 Except where otherwise indicated, all measurements are in points.
 	72 points = 1 inch
 	360 points = 127 milimeters

 PostScript is a registered trademark of Adobe Systems Incorporated.

 Copyright (C) 1999, Richard Urena.  All rights reserved.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.   You can find the GNU
 General Public License in the file COPYING in the source directory,
 or at http://www.gnu.org

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 ********************************************************************/

#include <stdio.h>
#include <errno.h>

#include "globals.h"
#include "defaults.h"
#include "types.h"

char *Copyright = 
	"Copyright (C) 1999, Richard R. Urena.  All rights reserved.";

int ExitCode = 0;		/* program termination status */
int AssumeDuplex = MPS__ASSUMEDUPLEX;	/* assume duplex printing? */
int FoldLong = MPS__FOLDLONG;	/* fold or truncate? */
int PageNumbers = MPS__PAGENUMBERS; /* print page numbers? */
int PageNo = MPS__FIRSTPAGENO; /* starting where? */
int TabStop = MPS__TABSTOP;    /* expand tabs to ... spaces */
char *EnvironmentVar = "MPSOPTIONS";   /* name of mps's environment var */

struct t_margins Margin = {
	{ MPS__MARGINLEFT, MPS__DEFAULTUNITS },
	{ MPS__MARGINRIGHT, MPS__DEFAULTUNITS },
	{ MPS__MARGINTOP, MPS__DEFAULTUNITS },
	{ MPS__MARGINBOTTOM, MPS__DEFAULTUNITS }
};

/*
 *	A couple of Courier fonts are defined.  If you want to add more:
 *		Choose monospace fonts only
 *		Make sure your PostScript interpreter has them, and that
 *		they can take the ISOLatin1 encoding vector.
 *		Provide the appropriate metrics as defined in the comments.
*/
struct t_fontmetric KnownFonts[] = {
	{"courier9",  "Courier", 5.4,  9.0, 4.6, 10.8 },
	{"courier10", "Courier", 6.0, 10.0, 5.1, 12.0 },
	{"courier12", "Courier", 7.2, 12.0, 6.12, 14.4 },
	{MPS__EMPTYSTRING, 0, 0, 0, 0 }	/* this must be the terminator */
};

struct t_fontmetric	*Font = &(KnownFonts[MPS__DEFAULTFONT]);


/*
 *	Now tell mps about the kinds of paper it knows about,
*/
struct t_paperinfo KnownPapers[] = {
	{"letter", 612,  792 },				/* 8.5 x 11 inches */
	{"legal",  612, 1008 },				/* 8.5 x 14 inches */
	{"a4",     595,  842 },				/* 210 x 297 mm */
	{MPS__EMPTYSTRING, 0, 0 }			/* this must be the terminator */
};

struct t_paperinfo	*Paper = &(KnownPapers[MPS__DEFAULTPAPER]);


/*
 *  what character to use as a continuation character
 *  to indicate when long lines are folded?
 *
 *  octal 273 is the gillemotright character (>>),
 *  defined as such both in the standard PostScript
 *  encoding, as well as in ISOLatin1
*/
char *MPS__CONTINUECHAR	= "\\273";


/*
 * conversion factors for various measurement units
 * (if you add more units, use only alphanumeric characters
 *  in their abbreviations)
*/
struct t_munits Measurement[] = {
	{72.0, "in" },			/* inches */
	{1.0, "pt" },			/* points */
	{2.8346457, "mm"},		/* milimeters */
	{28.346457, "cm"},		/* centimeters */
	{-1.0, MPS__EMPTYSTRING}	/* this must be the terminator */
};


/*
 * data structure for error processing
 *
*/
struct t_errmessage Error[MPS__ERRNITEMS] = {
	/* fatal errors */
	{ 0x10,	"Bad option (type mps --help)" },
	{ 0x10,	"Unknown font" },
	{ 0x10,	"Unknown paper" },
	{ 0x10,	"Bad page number" },
	{ 0x10,	"Bad tab stop" },
	{ 0x10,	"Bad or inconsistent margins" },
	{ 0x08,	MPS__EMPTYSTRING},			/* Out of memory */
	/* not-necessarily-fatal errors */
	{ 0x04, MPS__EMPTYSTRING},			/* can't open an input file */
	{ 0x02, MPS__EMPTYSTRING},			/* fgets error */
	{ 0x01, "Some extremely long lines were truncated"}
};


/* ------------------------------------------------------------------
   m p s _ s y s e r r o r
   basically, a wrapper around perror
   ------------------------------------------------------------------ */
void mps_syserror(enum t_errindex Index, int Syserrcode, char *Prefix)
{
	errno = Syserrcode;
	perror(Prefix);
	ExitCode |= (Error[Index].ExitCode);
}

/* ------------------------------------------------------------------
   m p s _ e r r o r
   notify errors peculiar to mps
   ------------------------------------------------------------------ */
void mps_error(enum t_errindex Index, char *Prefix)
{
	fprintf(stderr,"%s: %s\n",Prefix,Error[Index].Message);
	ExitCode |= (Error[Index].ExitCode);
}


/* ------------------------------------------------------------------
   V a l i d a M a r g i n s
   check the margins structure for validity
   this is called from the main program, after options have been
   processed
   ------------------------------------------------------------------ */
int ValidMargins(char *Progname)
{
	/* up to this point, margin measurements have existed in terms
	   of user "units".  convert them to points now
	*/
	Margin.Top.Measure *= Measurement[Margin.Top.Units].ConvFactor;
	Margin.Bottom.Measure *= Measurement[Margin.Bottom.Units].ConvFactor;
	Margin.Left.Measure *= Measurement[Margin.Left.Units].ConvFactor;
	Margin.Right.Measure *= Measurement[Margin.Right.Units].ConvFactor;

	/* they all must be non-negative */

	if( (Margin.Top.Measure<0.0) || (Margin.Bottom.Measure<0.0) ||
			 (Margin.Left.Measure<0.0) || (Margin.Right.Measure<0.0)  )	{
		mps_error(MPS__ERRBADMARGINS,Progname);
		return(0);
	}

	/* and they must leave a printable area */

	if(((Paper->Width-(Margin.Left.Measure+Margin.Right.Measure))
						 < Font->CharWidth) ||
			((Paper->Height-(Margin.Top.Measure+Margin.Bottom.Measure))
						 < Font->LineSkip) ) 							{
		mps_error(MPS__ERRBADMARGINS,Progname);
		return(0);
	}
	return(1);
}
